Global('Button', {})

function Button:Init(widget)
	self.widget = widget
	self.state = false
	self.activeEventHandlers = self:GetActiveEventHandlers()
	self.activeReactionHandlers = self:GetActiveReactionHandlers()
end

function Button:Toggle(state)
  if self.state ~= state then
  	if state then
			self:PlayEffectOut()
  	end
		common.ToggleEventHandlers(self.activeEventHandlers, state)
		common.ToggleReactionHandlers(self.activeReactionHandlers, state)
    self.state = state
  end
end

function Button:Show(visible)
	self.widget:Show(visible)
end

function Button:GetVariant()
	return self.widget:GetVariant()
end

function Button:SetVariant(variant)
	self.widget:SetVariant(variant)
end

function Button:PlayMoveEffect(before, after, ms, algorithm, addonName)
	self.widget:PlayMoveEffect(
		table.add(self.widget:GetPlacementPlain(), before or {}),
		table.add(self.widget:GetPlacementPlain(), after or {}),
		ms, algorithm or EA_MONOTONOUS_INCREASE, addonName)
end

function Button:PlayMoveEffectSequence(sequence)
	self:FinishMoveEffectSequence()
	self.moveSequence, self.moveSequenceStep = sequence, 1
	self:PlayMoveEffect(unpack(self.moveSequence[self.moveSequenceStep]))
end

function Button:FinishMoveEffectSequence()
	self.widget:FinishMoveEffect()
	self.moveSequence, self.moveSequenceStep = nil
end

function Button:PlayEffectIn()
	self:PlayMoveEffectSequence {
		{nil, {highPosX = 0}, 70}
	}
end

function Button:PlayEffectOut()
	self:PlayMoveEffectSequence {
		{nil, nil, 1000},
		{nil, {highPosX = -22}, 200}
	}
end

function Button:GetActiveEventHandlers()
	return {
		EVENT_EFFECT_FINISHED = function(event)
			if event.effectType == ET_MOVE and event.wtOwner and event.wtOwner:IsValid() and self.widget:IsEqual(event.wtOwner) then
				if self.moveSequence then
					self.moveSequenceStep = self.moveSequenceStep + 1
					if self.moveSequence[self.moveSequenceStep] then
						self:PlayMoveEffect(unpack(self.moveSequence[self.moveSequenceStep]))
					else
						self:FinishMoveEffectSequence()
					end
				end
			end
		end
	}
end

function Button:GetActiveReactionHandlers()
	return {
		btn_lck = function(reaction) -- button_pressed
			if self.widget:IsEqual(reaction.widget) then
				self:SetVariant(self:GetVariant() == 1 and 0 or 1)
			end
		end,
		btn_shft = function(reaction) -- button_over
			if self.widget:IsEqual(reaction.widget) then
				if reaction.active then
					self:PlayEffectIn()
				else
					self:PlayEffectOut()
				end
			end
		end
	}
end
